#!/usr/bin/env bash
#===============================================================
# MUSASHI bash script
#
# 商品マスターの作成
#
# Version :  2002/05/22 初期作成 by Y.Hamuro
#
# 入力ファイル : $inPath/jicfs6.xt
# 出力ファイル : $outPath/syo.xt
#
# 中分類が11,12,13,14のみに限定している。
# 細分類名に「その他」の文字が含まれるものは除外している。
#
# 出力ファイルのイメージ
#  商品 商品名 大分類 中分類 小分類 細分類 メーカー ブランド 仕入単価
#  0000001 スポーツドリンク8 1 14 1403 140323 0876 087603 373
#  0000002 きな粉1 1 11 1110 111011 0168 016804 399
#  0000003 動物性油脂1 1 11 1102 110207 0130 013004 338
#  0000004 醤油5 1 11 1101 110101 1398 139803 210
#  0000005 焼き豚1 1 11 1115 111509 0471 047104 310
#  0000006 天ぷら粉2 1 11 1110 111003 0484 048403 428
#  0000007 リキュール類4 1 14 1406 140681 0013 001303 288
#  0000008 豆腐2 1 11 1118 111801 0591 059102 305
#  0000009 マカロニ5 1 11 1112 111211 0739 073903 236
#  0000010 唐揚げ粉類1 1 11 1110 111007 0133 013304 138
#                              :
#===============================================================
inPath="../indat"
outPath="../outdat"

. ./debugEnv.sh

#---- タイトル
title="商品マスター"

#---- コメント
comment="人工データ"

#---- 乱数の種
RandSeed=10

#-------------------------------------------
#商品別大中小細分類＋仕入単価表を作成する
#-------------------------------------------
#jicfs細分類マスターから、大中小分類コードを生成
${dbg}xtsubstr -f 細分類:大分類 -R 1   -A -i $inPath/jicfs6.xt |
${dbg}xtsubstr -f 細分類:中分類 -R 1_2 -A |
${dbg}xtsubstr -f 細分類:小分類 -R 1_4 -A |

#中分類が11,12,13,14のみを選択
${dbg}xtselstr -f 中分類 -v 11,12,13,14 |

#細分類名に「その他」が含まれるものは除外
${dbg}xtsel -c 'not($細分類名=~"その他")' |

#各細分類につきいくつの商品を用意するかを正規分布に基づいた
#乱数によって決める
${dbg}xtcal -c "down(nrand(4,2,$RandSeed),1)" -a 乱数 |
${dbg}xtsel -c '$乱数>0' |

#与えられた商品数分だけ行を複写する
${dbg}xtduprec -f 乱数 |
${dbg}xtcut -f 乱数 -r |

#同じ細分類内で連番をふり、細分類名＋連番を商品名とする
${dbg}xtnumber -k 細分類 -a 番号 |
${dbg}xtcal -c 'cat($細分類名,$番号,"")' -a 商品名 |

#必要な項目を抜きだす
${dbg}xtcut -f 商品名,大分類,中分類,小分類,細分類 |

#各商品の仕入単価をランダムに決める
#ただし、５０円を下回る商品は除外する
${dbg}xtcal -c "down(nrand(298,100,$RandSeed),1)" -a 仕入単価 |
${dbg}xtsel -c '$仕入単価>=50' >xxa

#ここまでの処理で、以下のようなデータが出来上がっている
#  商品名 大分類 中分類 小分類 細分類 仕入単価
#  醤油1 1 11 1101 110101 377
#  醤油2 1 11 1101 110101 354
#  醤油3 1 11 1101 110101 200
#  醤油4 1 11 1101 110101 360
#  醤油5 1 11 1101 110101 210
#  砂糖1 1 11 1101 110103 411
#  砂糖2 1 11 1101 110103 332
#              :

#-------------------------------------------
#メーカマスターを作成する
#-------------------------------------------
#4桁のメーカーコードを2000行生成する
${dbg}xtmkdata -l 2000 -c 'fix(line(),4,"R","0")' -a メーカー |

#各メーカにつき数個のブランドを乱数で作成し、
#メーカー＋乱数２桁をブランドコードとする
${dbg}xtcal -c "down(nrand(4,2,$RandSeed),1)" -a 乱数 |
${dbg}xtsel -c '$乱数>0' |
${dbg}xtduprec -f 乱数 |
${dbg}xtcal -c 'cat($メーカー,fix(down(rand(1,$乱数,'$RandSeed'),1),2,"R","0"),"")' -a ブランド |
${dbg}xtcut -f メーカー,ブランド >xxb
#ここまでの処理で、以下のようなデータが出来上がっている
#  メーカー ブランド
#  0001 000104
#  0001 000102
#  0001 000105
#  0001 000101
#  0001 000105
#  0002 000204
#  0002 000204
#  0002 000201
#  0002 000203
#  0002 000203
#  0003 000304
#       :

#-------------------------------------------
#商品別大中小細分類＋仕入単価表に
#メーカーマスタをjoinする
#-------------------------------------------
#xxaとxxbをランダムに結合
${dbg}xtrand -a 乱数 -S $RandSeed -i xxa |
${dbg}xtsort -k 乱数 |
${dbg}xtnumber -a seq >xxaa

${dbg}xtrand -a 乱数 -S $RandSeed -i xxb |
${dbg}xtsort -k 乱数 |
${dbg}xtnumber -a seq >xxbb

${dbg}xtjoin -k seq -m xxbb -f メーカー,ブランド -i xxaa |
${dbg}xtcut -r -f 乱数 |

#-------------------------------------------
#仕上げ
#-------------------------------------------
#ランダムに並べかえて７桁の商品コードを作成する
${dbg}xtrand -S $RandSeed -a 乱数 |
${dbg}xtnumber -s 乱数%n -a 連番 |
${dbg}xtcal -c 'fix($連番,7,"R","0")' -a 商品 |

#必要な項目を抜きだし、商品順に並べかえ、ヘッダーを修正して終り
${dbg}xtcut -f 商品,商品名,大分類,中分類,小分類,細分類,メーカー,ブランド,仕入単価 |
${dbg}xtsort -k 商品 |
${dbg}xtheader -l "$title" -c "$comment" -o $outPath/syo.xt

rm -f xx*
#===============================================================
